#include <string.h>
#include <stdlib.h>

#include "image.h"
#include "number.h"

static SDL_Surface *digit[NUM_DIGITS];

/* 
 * free_digits() - free n digits
 */

static inline void free_digits(unsigned int n)
{
	unsigned int i;

	for(i = 0; i < n; i++) 
		SDL_FreeSurface(digit[i]);
}

/* 
 * init_numbers() - load number data 
 */

bool init_numbers(void)
{
	int i;
	char *file, *path;

	for(i = 0; i < NUM_DIGITS; i++) {

		file = get_number_image(i);
		if(!file) {
			fprintf(stderr, "Out of memory.\n");
			free_digits(i);
			return false;
		}

		path = data_file(file);
		if(!path) {
			fprintf(stderr, "Out of memory.\n");
			free_digits(i);
			return false;
		}
		free(file);
		
		digit[i] = load_image(path, true);
		if(!digit[i]) {
			fprintf(stderr, "Unable to load image %s: %s\n", path, SDL_GetError());
			free_digits(i);
			return false;
		}	

		free(path);
	}
	digit[i] = NULL;

	return true;
}


/*
 * free_numbers() - free number data
 */

void free_numbers()
{
	free_digits(NUM_DIGITS);
}

/* 
 * get_digit() - get the corresponding SDL_Surface for num
 */

SDL_Surface *num_get_digit(unsigned int num)
{
	if(num < 0 || num > NUM_DIGITS)
		return NULL;

	return digit[num];
}

/*
 * get_number_image() - returns the appropriate image file name for 
 * the number given.
 */

char *get_number_image(unsigned int num)
{
	char *img;
	
	if(num >= NUM_DIGITS)
		return NULL;

	switch(num) {
		case 0: img = strdup("zero.png");	break;
		case 1: img = strdup("one.png");	break;
		case 2:	img = strdup("two.png");	break;
		case 3:	img = strdup("three.png");	break;
		case 4:	img = strdup("four.png");	break;
		case 5:	img = strdup("five.png");	break;
		case 6:	img = strdup("six.png");	break;
		case 7:	img = strdup("seven.png");	break;
		case 8:	img = strdup("eight.png");	break;
		case 9:	img = strdup("nine.png");	break;
	}

	return img;
}

/*
 * get_digit() - return the dth digit in number.
 * 
 * The digits are like so (say number = 12345):
 *
 * 1   2   3   4   5
 * 0th 1st 2nd 3rd 4th
 *
 * So if we want to get the '4' from number we 
 * would call get_digit like 
 * 
 * num = get_digit(number, 3);
 */

int get_digit(int number, int d)
{
	int digits;
	int i;
	int mod, div;

	digits = num_digits(number);

	if(d > digits) 
		return -1;

	mod = 10;
	div =  1;

	for(i = (digits - 1); i > d; i--)
		mod *= 10;

	div = mod / 10;

	return (number % mod) / div;
}

/* 
 * num_digits() - returns the number of digits in 'number'
 */

inline unsigned int num_digits(unsigned int number)
{
	int result = 0;

	do {
		++result;
	} while(number /= 10);

	return result;
}
