#include "timer.h"
#include "number.h"
#include "image.h"
#include "video.h"

bool PAUSE_TIMER;

static time_t init_time;
static time_t total_time;
static time_t paused_at;

object colon;
object numbers[2];

bool load_timer_data();

/*
 * init_timer() - initialize the timer
 */

bool init_timer()
{
	if(!load_timer_data())
		return false;

	numbers[0].x = 175;
	numbers[0].y = screen->h - numbers[0].image->h - 2;

	colon.x = numbers[0].x + numbers[0].image->w + 2;
	colon.y = screen->h - colon.image->h - 2;

	numbers[1].x = colon.x + colon.image->w + 2;
	numbers[1].y = screen->h - numbers[1].image->h - 2;

	numbers[2].x = numbers[1].x + numbers[1].image->w + 2;
	numbers[2].y = screen->h - numbers[2].image->h - 2;

	init_time = time(NULL);

	PAUSE_TIMER = false;

	return true;
}

/* 
 * load_timer_data() - load the timer data
 */

bool load_timer_data()
{
	int i;

	colon.image = load_image(DATAFILE("colon.png"), true);
	if(!colon.image) {
		fprintf(stderr, "Unable to load image %s: %s\n", DATAFILE("colon.png"),
			SDL_GetError());
		return false;
	}

	for(i = 0; i < 3; i++) 
		numbers[i].image = num_get_digit(0);	

	return true;
}

/*
 * free_timer() - free timer data 
 */

void free_timer()
{
	SDL_FreeSurface(colon.image);
}

/*
 * draw_timer() - draw the timer
 */

void draw_timer()
{
	int i;

	for(i = 0; i < 3; i++)
		draw_object(&numbers[i]);

	draw_object(&colon);
}

/*
 * erase_timer() - erase the timer
 */

void erase_timer()
{
	int i;

	for(i = 0; i < 3; i++)
		erase_object(&numbers[i]);

	erase_object(&colon);
}

/* 
 * pause_timer() - pause the timer
 */

void pause_timer()
{
	PAUSE_TIMER = true;
	paused_at = time(NULL);
}

/*
 * unpause_timer() - unpause the timer
 */

void unpause_timer()
{
	PAUSE_TIMER = false;
	init_time += time(NULL) - paused_at;	
}

/*
 * reset_timer() - reset the timer to 0
 */

void reset_timer()
{
	init_time = time(NULL);
	total_time = 0;
}

/*
 * update_timer() - update the timer
 */

void update_timer()
{
	int seconds, minutes;
	int s1, s2;

	if(PAUSE_TIMER) return;

	total_time = time(NULL) - init_time;

	seconds = total_time % 60;
	minutes = total_time / 60;

	s1 = seconds / 10;
	s2 = seconds % 10;

	numbers[0].image = num_get_digit(minutes);

	numbers[1].image = num_get_digit(s1);
	numbers[2].image = num_get_digit(s2);
}
